/* layer3.c */

#include "types.h"

extern void (*L3_window_filter_subband)(unsigned long **buffer, long s[SBLIMIT], int k);
extern void (*L3_mdct_sub)(long sb_sample[2][3][18][SBLIMIT],
                 long mdct_freq[2][2][samp_per_frame2]);

int *scalefac_band_long;

/* MPEG1 Table B.8 and MPEG2 Table B.1 -- Layer III scalefactor bands.
 * Index into this using a method such as:
 * idx  = fr_ps->header->sampling_frequency + (fr_ps->header->version * 3)
 */
static int sfBandIndex[2][3][23] =
{
  {
    /* Table B.2.b: 22.05 kHz */
    { 0,6,12,18,24,30,36,44,54,66,80,96,116,140,168,200,238,284,336,396,464,522,576 },
    /* Table B.2.c: 24 kHz */
    { 0,6,12,18,24,30,36,44,54,66,80,96,114,136,162,194,232,278,330,394,464,540,576 },
    /* Table B.2.a: 16 kHz */
    { 0,6,12,18,24,30,36,44,45,66,80,96,116,140,168,200,238,248,336,396,464,522,576 }
  },
  {
    /* Table B.8.b: 44.1 kHz */
    { 0,4,8,12,16,20,24,30,36,44,52,62,74,90,110,134,162,196,238,288,342,418,576 },
    /* Table B.8.c: 48 kHz */
    { 0,4,8,12,16,20,24,30,36,42,50,60,72,88,106,128,156,190,230,276,330,384,576 },
    /* Table B.8.a: 32 kHz */
    { 0,4,8,12,16,20,24,30,36,44,54,66,82,102,126,156,194,240,296,364,448,550,576 }
  }
};

/*
 * update_status:
 * --------------
 */
static void update_status(int frames_processed)
{
  printf("\015[%d] %ld%%",
    frames_processed,(frames_processed*100)/config.mpeg.total_frames);
}

/*
 * L3_compress:
 * ------------
 */
void L3_compress(void)
{
  int             frames_processed;
  int             channel;
  int             i;
  int             gr;
  unsigned long   *buffer[2];
  long          remainder;
  long          bytes_per_frame;
  long          lag;
  int             mean_bits;
  int             sideinfo_len;
  static int      l3_enc[2][2][samp_per_frame2];
  static long     l3_sb_sample[2][3][18][SBLIMIT];
  static long     mdct_freq[2][2][samp_per_frame2];
  static L3_side_info_t  side_info;

  open_bit_stream(config.outfile);

  config.mpeg.samples_per_frame = samp_per_frame;
  frames_processed = lag = 0;
  sideinfo_len = (config.wave.channels==1) ? 168 : 288;
  scalefac_band_long = sfBandIndex[config.mpeg.type][config.mpeg.samplerate_index];
  config.mpeg.total_frames =  /* include any remainder */
    (config.wave.total_samples + config.mpeg.samples_per_frame - 1) / config.mpeg.samples_per_frame;

  { /* find number of whole bytes per frame and the remainder */
    long x = config.mpeg.samples_per_frame * config.mpeg.bitr * (1000/8);
    bytes_per_frame = x / config.wave.samplerate;
    remainder  = x % config.wave.samplerate;
  }

  printf("%ld frames\n",config.mpeg.total_frames);

  while((buffer[1] = buffer[0] = wave_get()) != 0)
  {
    frames_processed++;
    if(((frames_processed & 7)==0) || (frames_processed >= config.mpeg.total_frames))
      update_status(frames_processed);

    /* sort out padding */
    config.mpeg.padding = (lag += remainder) >= config.wave.samplerate;
    if (config.mpeg.padding)
      lag -= config.wave.samplerate;
    config.mpeg.bits_per_frame = 8*(bytes_per_frame + config.mpeg.padding);
    mean_bits = (config.mpeg.bits_per_frame - sideinfo_len)>>1; /* per granule (1 or 2 channels) */

    /* polyphase filtering */
    for(gr=0;gr<2;gr++)
      for(channel=config.wave.channels; channel--; )
        for(i=0;i<18;i++)
          (*L3_window_filter_subband)(&buffer[channel], &l3_sb_sample[channel][gr+1][i][0] ,channel);

    /* apply mdct to the polyphase output */
    (*L3_mdct_sub)(l3_sb_sample, mdct_freq);

    /* bit and noise allocation */
    L3_iteration_loop(mdct_freq, &side_info, l3_enc, mean_bits);

    /* write the frame to the bitstream */
    L3_format_bitstream(l3_enc, &side_info);
  }
  close_bit_stream();
}


